;;; *******************************************************************
;;; * Autor         : Dipl. Ing. Jan Tappenbeck                       *
;;; * Projektname   : Kreisbogen in zwei ungleiche Bgen zerlegen     *
;;; * fr AutoCAD   : R14/2000/2000i/2002, Map 4/5                    *
;;; *                 LT2002 in Verbindung mit LT-Extender            *
;;; *******************************************************************
;;; * Versionsnummer: 1.0                                             *
;;; * Datei         : c2a.LSP                                         *
;;; * zugehrige DCL: - entfllt -                                    *
;;; * zugehrige SLD: - entfllt -                                    *
;;; * Sprache       : deutsch / international                         *
;;; * nderungen    : ./.                                             *
;;; *******************************************************************
;;; * Kurzbeschreibung:                                               *
;;; * Kreisbogen in zwei ungleiche Bgen zerlegen                     *
;;; *******************************************************************
;;; * Globale Variablen                                               *
;;; * - entfllt -                                                    *
;;; *******************************************************************
;;; * (c) Jan Tappenbeck, 08/2002, Germany                            *
;;; * jan.tappenbeck@t-online.de                                      *
;;; *******************************************************************
;;; *                                                                 *
;;; * Lizenz:                                                         *
;;; * Das Programm wird als Freeware-Version freigegeben !            *
;;; *                                                                 *
;;; * Fr Schden, die durch die Anwendung dieses Programmes          *
;;; * entstehen, kann keine Haftung bernommen werden !               *
;;; *                                                                 *
;;; *******************************************************************
;;; *                                                                 *
;;; * Disclaimer:                                                     *
;;; * This software is provided "as is" without express or            *
;;; * implied warranty.                                               *
;;; *                                                                 *
;;; *******************************************************************

(setq *c2a#release* "2002-08-13")

(defun c:c2a				;Funktion zum Aufbrechen eines Kreises in zwei ungleiche Kreisbgen
	     (/			 kreis	;gewhlter Kreis
	      eliste			;Entity-Liste
	      gc_eliste			;Gruppencode
	      circle_center_2D		;Kreismittelpunkt - 2D
	      circle_center_3D		;Kreismittelpunkt - 3D
	      p1			;erster Bruchpunkt
	      p2			;zweiter Bruchpunkt
	      min_dist			;min. Abstand damit Punkte nicht identisch
	      min_arc			;min. Radialabstand damit Bogenpunkte nicht den gleichen Kreispunkt definieren
	      arc_p1			;Winkel zum Punkt 1 vom Zentrum
	      arc_p2			;Winkel zum Punkt 2 vom Zentrum
	      arc_radius		;Radius
	      winkel#1			;Richtung zum Punkt 1 vom Mittelpunkt
	      winkel#2			;Richtung zum Punkt 2 vom Mittelpunkt
	      2pi			;Vollkreis im Bogenmass
	      kreis_layer		;Layer des Kreiselementes
	      lock_flag			;gesperrt oder nicht
	      del_answer		;Rckmeldung der Abfrage
	      layer_eliste		;Daten des Layers vom Kreis
	      ;;nachfolgend die Eigenschaften des Kreises
	      #layer			;Layer
	      delta_arc			;Winkelunterschied fr Bogen 2
	      #color			;Farbe
	      #ltype			;Linientyp
	      #lweight			;Linienstrke
	      #layout			;Layout
	      #center			;zentrum
	      #radi			;Radius
	      #thinkness		;Objekterhebung
	      #ltscale			;Linienmusterfaktor
	      #normale			;fr 3D
	      #space			;0/[entfllt] ... Modellbereich / 1 ... Papierbereich
	      draw_arc#plot_objID	;Plotstyle_Name_ObjID
	      #plot_type		;Plotstyle_Name_Type
	      ;;bergabe-Variablen fr die Funktion zum erstellen der Bgen
	      jtc#draw_arc#layer	;Layer
	      jtc#draw_arc#color	;Farbe
	      jtc#draw_arc#ltype	;Linientyp
	      jtc#draw_arc#lweight	;Linienstrke
	      jtc#draw_arc#layout	;Layout
	      jtc#draw_arc#center	;zentrum
	      jtc#draw_arc#radi		;Radius
	      jtc#draw_arc#start	;startwinkel
	      jtc#draw_arc#end		;endwinkel
	      jtc#draw_arc#thinkness	;Objekterhebung
	      jtc#draw_arc#ltscale	;Linienmusterfaktor
	      jtc#draw_arc#normale	;fr 3D
	      jtc#draw_arc#space	;0/[entfllt] ... Modellbereich / 1 ... Papierbereich
	      jtc#draw_arc#plot_objID	;Plotstyle_Name_ObjID
	      jtc#draw_arc#plot_type	;Plotstyle_Name_Typee
	      jtc#draw_arc*lst*	 c2a#msg1	    c2a#msg2
	      c2a#msg3		 c2a#msg4	    c2a#msg5
	      c2a#msg6		 c2a#msg7	    c2a#msg8
	      c2a#msg9		 c2a#msg10	    c2a#msg11
	      c2a#msg12		 c2a#msg13	    c2a#msg14
	      c2a#msg100	 c2a#msg900	    c2a#msg901
	      )

  (modes				;speichern der Systemvariablen
    '("CMDECHO"	"BLIPMODE" "ANGDIR" "ANGBASE" "AUNITS" ;Bogenma
      "LUNITS")
    )
  (SETVAR "CMDECHO" 0)
  (COMMAND "_.UNDO" "_Begin")

  (SETQ
    old_err *error*			;setzen der internen Fehlerroutine
    *error* c2a#_err
    )					;end-setq

  (c2a$language)			;Sprachmodul

  (SETVAR "BLIPMODE" 0)
  (setvar "ANGDIR" 0)
  (setvar "ANGBASE" 0)
  (setvar "AUNITS" 3)			;Bogenma
  (setvar "LUNITS" 2)

  (princ c2a#msg1)			;berschrift


  (setq	min_dist 0.0001
	min_arc	0.00001
	2pi (* 2 pi)			;Vollkreis
	)				;end-setq

  (if (not (setq kreis (entsel c2a#msg2) ;Kreis whlen
		 )			;end-setq
	   )				;end-not
    (princ c2a#msg3)			;kein Element
    (progn
      (setq eliste (entget (car kreis))
	    )
      (setq
	gc_eliste    (cdr (assoc 0 eliste)) ;Elementyp
	kreis_layer  (cdr (assoc 8 eliste)) ;Layername
	layer_eliste (tblsearch "LAYER" kreis_layer)
	lock_flag    (CDR (ASSOC 70 layer_eliste))
	)				;end-setq

      (if (/= gc_eliste "CIRCLE")
	(princ c2a#msg4)		;no Kreis
	(progn				; falsches Element
	  (setq	circle_center_3D (cdr (assoc 10 eliste))
		circle_center_2D
				 (list
				   (cadr (assoc 10 eliste)) ;Rechtswert
				   (cadr (cdr (assoc 10 eliste)))
					;Hochwert
				   )	;end-list
		arc_radius	 (cdr (assoc 40 eliste)) ;Kreisradius
		)			;end-setq



	  (if (not (setq p1 (getpoint circle_center_2D
				      c2a#msg5 ;1. Bruchpunkt whlen
				      )	;end-getpoint
			 )		;end-setq
		   )			;end-not
	    (princ c2a#msg100)		;benutzerdefininiertes Ende
	    (progn

	      (if (< (distance circle_center_2D p1) min_dist)
		(princ c2a#msg6)	;1. Bruch ident. Center
		(progn
		  (grdraw circle_center_2D p1 2)
		  (if (not (setq p2 (getpoint circle_center_2D
					      c2a#msg7 ;2. Bruch
					      ) ;end-getpoint
				 )	;end-setq
			   )		;end-not
		    (princ c2a#msg100)	;benutzerdef. Ende
		    (progn

		      (if (< (distance circle_center_2D p2) min_dist)
			(princ
			  c2a#msg8	;2. Bruch ident. Center
			  )
			(progn
			  (setq	arc_p1 (angle circle_center_2D p1)
				arc_p2 (angle circle_center_2D p2)
				)	;end-setq

			  (if (< (abs	;Prfung auf Winkelidentitt
				   (- arc_p1 arc_p2)
				   )	;end-abs
				 min_arc
				 )
			    (princ c2a#msg9) ;1. und 2. Bruch ident.
			    (progn

			      (if (< arc_p1 arc_p2)
					;Winkeldefinition fr das Zeichnen der Bgen
				(setq
				  winkel#1 arc_p1
				  winkel#2 arc_p2
				  )
				(setq
				  winkel#1 arc_p2
				  winkel#2 arc_p1
				  )
				)	;endif

			      (setq delta_arc
					;Winkelunterschied fr Bogen 2
				     (- 2pi (- winkel#2 winkel#1))
				    )	;end-setq
			      ;; nachfolgender Quellcode ist vielleicht etwas umstndlich - kann dafr aber anderweitig universeller eingesetzt werden
			      ;;schlielich ist der Mensch ein faules Tier
			      (setq
				#layer (cdr (assoc 8 eliste)) ;Layer
				#color (cdr (assoc 62 eliste)) ;Farbe
				#ltype (cdr (assoc 6 eliste))
					;Linientyp
				#lweight (cdr (assoc 370 eliste))
					;Linienstrke
				#layout	(cdr (assoc 410 eliste))
					;Layout
				#center	(cdr (assoc 10 eliste))
					;zentrum
				#radi (cdr (assoc 40 eliste))
					;Radius
				#start (cdr (assoc 50 eliste))
					;startwinkel
				#end (cdr (assoc 51 eliste))
					;endwinkel
				#thinkness
				 (cdr (assoc 39 eliste))
					;Objekterhebung
				#ltscale (cdr (assoc 48 eliste))
					;Linienmusterfaktor
				#normale (cdr (assoc 210 eliste))
					;fr 3D
				#space (cdr (assoc 67 eliste))
					;0/[entfllt] ... Modellbereich / 1 ... Papierbereich
				draw_arc#plot_objID
				 (cdr
				   (assoc 380 eliste)
				   )	;Plotstyle_Name_ObjID
				#plot_type
				 (cdr (assoc 390 eliste))
					;Plotstyle_Name_Type
				)	;end-setq

			      ;;Bogen 1 zeichnen mit winkel#1 und winkel#2

			      (jtc#draw_arc ;zeichnet den Bogen
				#layer	;Layer
				#color	;Farbe
				#ltype	;Linientyp
				#lweight ;Linienstrke
				#layout	;Layout
				#center	;zentrum
				#radi	;Radius
				winkel#1 ;startwinkel
				winkel#2 ;endwinkel
				#thinkness ;Objekterhebung
				#ltscale ;Linienmusterfaktor
				#normale ;fr 3D
				#space	;0/[entfllt] ... Modellbereich / 1 ... Papierbereich
				draw_arc#plot_objID
					;Plotstyle_Name_ObjID
				#plot_type ;Plotstyle_Name_Type
				)	;end-jtc#draw_arc

			      ;;Bogen 2 zeichnen mit winkel#2 und winkel#1
			      (jtc#draw_arc ;zeichnet den Bogen
				#layer	;Layer
				#color	;Farbe
				#ltype	;Linientyp
				#lweight ;Linienstrke
				#layout	;Layout
				#center	;zentrum
				#radi	;Radius
				winkel#2 ;startwinkel
				(+ winkel#2 delta_arc) ;endwinkel
				#thinkness ;Objekterhebung
				#ltscale ;Linienmusterfaktor
				#normale ;fr 3D
				#space	;0/[entfllt] ... Modellbereich / 1 ... Papierbereich
				draw_arc#plot_objID
					;Plotstyle_Name_ObjID
				#plot_type ;Plotstyle_Name_Type
				)	;end-jtc#draw_arc
			      (IF (> (LOGAND lock_flag 4) 0)
					;prfen, ob Element gesperrt
				(princ
				  c2a#msg10 ;Quellobjekt gesperrt
				  )
				(progn

				  (initget c2a#msg11)
				  (setq	del_answer
					 (getkword
					   c2a#msg12
					;Quellobjekt lschen ?
					   )
					) ;end-setq


				  (if (or
					(= del_answer c2a#msg13) ;Ja
					(not del_answer)
					) ;end-or
				    (progn
				      (entdel (car kreis))
				      (princ c2a#msg14)
					;Objekt gelscht
				      )	;end-progn
				    )	;endif
				  )	;end-progn
				)	;endif - logand






			      )		;end-progn - identisch p2=p1
			    )		;endif - identisch p2=p1




			  )		;end-progn - identisch p2=m
			)		;endif - identisch p2=m


		      )			;end-progn - kein p2
		    )			;endif -  - kein p2


		  )			;end-progn - identisch p1=m
		)			;endif - identisch p1=m


	      )				;end-progn - kein p1
	    )				;endif -  - kein p1
	  )				;end-progn - falsches Element
	)				;endif - falsches Element

      )					;end-progn - keine Element gewhlt
    )					;endif - kein Element gewhlt

  (command "_redraw")
  (SETQ *error* old_err)
  (COMMAND "_.UNDO" "_End")
  (moder)
  (PRINC)
  )




(DEFUN moder ()				; Rcksetzen der Systemvariablen - Unterprogramm
  (REPEAT (LENGTH mlst)
    (SETVAR (CAAR mlst) (CADAR mlst))
    (SETQ mlst (CDR mlst))
    )
  )

(DEFUN modes (a)			;speichern der Systemvariablen
  (SETQ mlst '())
  (REPEAT (LENGTH a)
    (SETQ mlst (APPEND mlst (LIST (LIST (CAR a) (GETVAR (CAR a))))))
    (SETQ a (CDR a))
    )
  )

(DEFUN c2a#_err	(s)			;interne Fehlerroutine
  (IF (/= s c2a#msg900)
    (PRINC (STRCAT c2a#msg901 s))
    )
  (command "_redraw")
  (SETQ *error* old_err)
  (COMMAND "_.UNDO" "_End")
  (COMMAND "_u")
  (moder)
  (PRINC)
  )

(defun jtc#draw_arc			;zeichnet den Bogen
		    (jtc#draw_arc#layer	;Layer
		     jtc#draw_arc#color	;Farbe
		     jtc#draw_arc#ltype	;Linientyp
		     jtc#draw_arc#lweight ;Linienstrke
		     jtc#draw_arc#layout ;Layout
		     jtc#draw_arc#center ;zentrum
		     jtc#draw_arc#radi	;Radius
		     jtc#draw_arc#start	;startwinkel
		     jtc#draw_arc#end	;endwinkel
		     jtc#draw_arc#thinkness ;Objekterhebung
		     jtc#draw_arc#ltscale ;Linienmusterfaktor
		     jtc#draw_arc#normale ;fr 3D
		     jtc#draw_arc#space	;0/[entfllt] ... Modellbereich / 1 ... Papierbereich
		     jtc#draw_arc#plot_objID ;Plotstyle_Name_ObjID
		     jtc#draw_arc#plot_type ;Plotstyle_Name_Type
		     /			;Liste fr den neuen Bogen
		     jtc#draw_arc$clst#gc ;Gruppencode
		     jtc#draw_arc$clst#data ;Gruppenwert
		     )


  (setq
    jtc#draw_arc*lst*
     (list (cons 0 "ARC")		;Elementtyp
	   )				;end-list
    )					;end-setq

  (jtc#draw_arc$clst 8 jtc#draw_arc#layer) ;Layer
  (jtc#draw_arc$clst 6 jtc#draw_arc#ltype) ;Linientyp
  (jtc#draw_arc$clst 10 jtc#draw_arc#center) ;zentrum
  (jtc#draw_arc$clst 39 jtc#draw_arc#thinkness) ;Objekterhebung
  (jtc#draw_arc$clst 40 jtc#draw_arc#radi) ;Radius
  (jtc#draw_arc$clst 48 jtc#draw_arc#ltscale) ;Linienmusterfaktor
  (jtc#draw_arc$clst 50 jtc#draw_arc#start) ;startwinkel
  (jtc#draw_arc$clst 51 jtc#draw_arc#end) ;Endwinkel
  (jtc#draw_arc$clst 62 jtc#draw_arc#color) ;Farbe
  (jtc#draw_arc$clst 67 jtc#draw_arc#space)
					;0/[entfllt] ... Modellbereich / 1 ... Papierbereich
  (jtc#draw_arc$clst 210 jtc#draw_arc#normale) ;fr 3D
  (jtc#draw_arc$clst 370 jtc#draw_arc#lweight)
					;Linienstrke, wenn von Layer, dann nicht vorhanden
  (jtc#draw_arc$clst 380 jtc#draw_arc#plot_type)
					;Plotstyle_Name_Type
  (jtc#draw_arc$clst 390 jtc#draw_arc#plot_objID)
					;Plotstyle_Name_ObjID
  (jtc#draw_arc$clst 410 jtc#draw_arc#layout)
					;Modell oder Layoutname

  (entmake jtc#draw_arc*lst*)
  )




(defun jtc#draw_arc$clst		;Erweitert Liste der Elementparameter
			 (jtc#draw_arc$clst#gc ;Gruppencode
			  jtc#draw_arc$clst#data ;Gruppenwert
			  /
			  )
  (if jtc#draw_arc$clst#data
    (setq jtc#draw_arc*lst*
	   (append jtc#draw_arc*lst*
		   (list (cons jtc#draw_arc$clst#gc jtc#draw_arc$clst#data)
					;Linientyp, wenn von Layer, dann nicht vorhanden
			 )		;end-list
		   )			;end-append
	  )				;end-setq
    )					;endif
  )					;end-defun - jtc#draw_arc$clst


(defun c2a$language
		    (/ #language)


  (setq #language (getvar "locale"))
  (cond
    ((= #language "DEU")
     (c2a$german)
     )
    (T
     (c2a$englisch)
     )
    )					;end-cond

  )					;end-defun


(defun c2a$german			;Men DEUTSCH
		  (/)
  (setq
    c2a#msg1   "\nKreisbogen in zwei ungleiche Bgen zerlegen"
    c2a#msg2   "\nKreis whlen: "
    c2a#msg3   "\nkein Element gewhlt !"
    c2a#msg4   "\nSie haben keinen KREIS gewhlt !"
    c2a#msg5   "\n1. Bruchpunkt whlen: "
    c2a#msg6   "\n1. Bruchpunkt identisch mit Kreismittelpunkt !"
    c2a#msg7   "\n2. Bruchpunkt whlen: "
    c2a#msg8   "\n2. Bruchpunkt identisch mit Kreismittelpunkt !"
    c2a#msg9   "\n1. und 2. Bruchpunkt definieren denselben Kreispunkt !"
    c2a#msg10  "\nQuellelement kann nicht gelscht werden !"
    c2a#msg11  "Ja Nein"
    c2a#msg12  "\nKreis lschen ? [Ja/Nein] <J>: "
    c2a#msg13  "Ja"
    c2a#msg14  "\nQuellkreis gelscht !"

    c2a#msg100 "\nbenutzerdefinierter Programmabbruch !"

    c2a#msg800 "\n(c) Jan Tappenbeck, 8/2002"
    c2a#msg801 "\nProgrammstart: C2A"

    c2a#msg900 "Funktion abgebrochen"
    c2a#msg901 "\nFehler: "
    )					;setq
  )					;end-defun - c2a$german

(defun c2a$englisch			;Men ENGLISCH
		    (/)
  (setq
    c2a#msg1   "\nSplit circle into arc"
    c2a#msg2   "\nselect circle: "
    c2a#msg3   "\nno element select !"
    c2a#msg4   "\nselected elemente is not a circle !"
    c2a#msg5   "\n1. breakpoint: "
    c2a#msg6   "\n1. breakpoint identity to circle-center"
    c2a#msg7   "\n2. breakpoint: "
    c2a#msg8   "\n2. breakpoint identity to circle-center"
    c2a#msg9   "\n1. breakpoint identity to 2. breakpoint"
    c2a#msg10  "\ncircle can not delete - lock !"
    c2a#msg11  "Yes No"
    c2a#msg12  "\ncircle delete ? [Yes/No] <Y>: "
    c2a#msg13  "Yes"
    c2a#msg14  "\ncircle delete !"

    c2a#msg100 "\nexit by user !"

    c2a#msg800 "\n(c) Jan Tappenbeck, 8/2002"
    c2a#msg801 "\nstart: c2a"
    c2a#msg802 (strcat "Release " *c2a#release*)
    
    c2a#msg900 "function cancel"
    c2a#msg901 "\nerror: "
    )					;setq
  )					;end-defun - c2a$englisch

(c2a$language)				;Sprachmodul
(princ c2a#msg1)			;berschrift
(princ c2a#msg800)			;Copyright
(princ c2a#msg801)			;Programmstart
(princ c2a#msg802);Version
(princ)
 ;|Visual LISP Format Options
(72 2 40 1 nil "end of " 60 9 0 0 0 T T nil T)
;*** KEINEN Text unterhalb des Kommentars hinzufgen! ***|;
